<?php
declare (strict_types=1);

namespace app\command;

use app\admin\model\OrderImportFromC as OIFCModel;
use think\console\Command;
use think\console\Input;
use think\console\input\Argument;
use think\console\input\Option;
use think\console\Output;
use think\Exception;
use think\facade\Db;
use think\facade\Validate;

//C端订单导入-数据解析
class ImportOrderFromCAnalysisData extends Command
{
    protected function configure()
    {
        // 指令配置
        $this->setName('analysisData')
            ->setDescription('批量解析C端导入的订单数据');
    }

    //【脚本1】解析用户导入的数据
    protected function execute(Input $input, Output $output)
    {
        //解析信息存到extend表,其中customer_code字段根据订单编号去customer_info查，其他根据商品编号查询

        Db::startTrans();
        try {

            $order_import_from_c_db = Db::name('order_import_from_c');

            $order_import_from_c_extend_db = Db::name('order_import_from_c_extend');

            $rs = $order_import_from_c_db
                ->where(['is_del' => OIFCModel::$is_del_normal, 'status' => OIFCModel::$status_wait_validate])
                ->find();

            if (empty($rs)) throw new Exception('无符合条件的记录');

            //1.数据校验
            $val = Validate::rule([
                'platform_code|平台订单号' => 'require',
                'platform_time|平台下单时间' => 'require|date',
                'sale_source|销售渠道' => 'require|checkSaleSource:',
                'plat_code|平台商品编号' => 'require',
                'price|单价' => 'require|float',
                'num|数量' => 'require|number|gt:0',
                'before_discount_all_price|总金额' => 'require|float',
                'mode|收费模式' => 'require|in:卡券支付,现金支付,混合支付',
                'contactor|收货人' => 'require',
                'mobile|联系电话' => 'require',
                'addr|联系地址' => 'require',
            ]);

            //校验销售渠道
            $val->extend('checkSaleSource', function ($val) {
                $temp = Db::name('platform_source')
                    ->field('id')
                    ->where(['source' => $val, 'is_del' => 0])
                    ->find();
                return empty($temp) ? '销售渠道不存在' : true;
            });

            $update_c_data = [
                'updatetime' => date('Y-m-d H:i:s'),
                'updateid' => 0,
                'updater' => 'system',
            ];
            if (!$val->check($rs)) {
                $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                $update_c_data['remark'] = '数据错误，' . $val->getError();
            }

            $extend_insert_data = [
                'order_import_from_c_id' => $rs['id'],
                'createrid' => 0,
                'creater' => 'system',
                'discount_reason' => $rs['activity_name'],
                'platform_code' => $rs['platform_code'],
                'po_code' => $rs['po_code'],
                'platform_time' => $rs['platform_time'],
                'price' => $rs['price'],
                'num' => $rs['num'],
                'before_discount_all_price' => $rs['before_discount_all_price'],
                'order_remark' => $rs['order_remark'],
                'contactor' => $rs['contactor'],
                'mobile' => $rs['mobile'],
                'addr' => $rs['addr'],
                'type' => 1,//1系统解析，2用户确认
                'addtime' => date('Y-m-d H:i:s'),
                'sale_source' => $rs['sale_source'],
                'mode' => $rs['mode'],

            ];


            //销售渠道
//            $extend_insert_data['sale_source'] = Db::name('platform_source')
//                ->where(['source' => $rs['sale_source'], 'is_del' => 0])
//                ->value('source', '');

            //组织extend的数据
            $good_platform_info = Db::name('good_platform')
                ->field('id,spuCode,platform_code,skuCode,plat_code')
                ->where(['is_del' => 0])
                ->where('plat_code', $rs['plat_code'])
                ->find();

            //商品信息
            if (!empty($good_platform_info)) {
                $extend_insert_data['spuCode'] = $good_platform_info['spuCode'];
                $extend_insert_data['platform_codes'] = $good_platform_info['platform_code'];
                $extend_insert_data['plat_code'] = $good_platform_info['plat_code'];

                $good_info = Db::name('good')
                    ->field('id,good_name,companyNo,supplierNo')
                    ->where(['is_del' => 0])
                    ->where('spuCode', $extend_insert_data['spuCode'])
                    ->find();
                if (!empty($good_info)) {
                    $extend_insert_data['companyNo'] = $good_info['companyNo'];
                    $extend_insert_data['supplierNo'] = $good_info['supplierNo'];
                    $extend_insert_data['good_name'] = $good_info['good_name'];
                } else {
                    $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                    $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',在上线商品表中查不到spuCode对应的商品信息' : '在上线商品表中查不到spuCode对应的商品信息';
                }

                //商品价格和总价
                $good_ladder = Db::name("good_ladder")
                    ->field('id,sale_price')
                    ->where(["skuCode" => $good_platform_info['skuCode'], "is_del" => 0, "status" => 1])
                    ->where([["min_num", "<=", $rs['num']]])
                    ->order("min_num desc")
                    ->find();
                if (empty($good_ladder)) {
                    $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                    $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',找不到商品售价' : '找不到商品售价';
                } else {

                    if ($rs['price'] < $good_ladder['sale_price']) {
                        $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                        $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',售价低于系统标准价' . $good_ladder['sale_price'] : '售价低于系统标准价' . $good_ladder['sale_price'];
                    }

                    if (($rs['num'] * $rs['price']) != $rs['before_discount_all_price']) {
                        $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                        $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',导入总价错误' : '导入总价错误';
                    }

                }

            } else {
                $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',在商品平台表中查不到平台商品编号对应的spuCode' : '在商品平台表中查不到平台商品编号对应的spuCode';
            }

            //活动信息
            if ($rs['activity_name']) {
                $activity_info = Db::name('good_activity')
                    ->field('id,activity_name,activity_code')
                    ->where(['activity_name' => $rs['activity_name'], 'is_del' => 0])
                    ->find();
                if (empty($activity_info)) {
                    $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                    $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',该活动不存在' : '该活动不存在';
                } else {
                    $extend_insert_data['activity_code'] = $activity_info['activity_code'];
                    $extend_insert_data['activity_name'] = $activity_info['activity_name'];
                }

            }

            //收费模式
//            $extend_insert_data['mode'] = in_array($rs['mode'], ['卡券支付', '现金支付', '混合支付']) ? $rs['mode'] : '';

            //企业客户
            $co = Db::name('good_platform')
                ->alias('gp')
                ->leftJoin('platform p', 'p.id=gp.platform_code AND p.is_del=0')
                ->leftJoin('customer_org1 co', 'co.name=p.platform_name AND co.is_del=0')
                ->where('gp.plat_code', $rs['plat_code'])
                ->field('co.id,co.name')->find();

            if ($co) {
                $customer_code = Db::name('customer_info')
                    ->where(['itemid' => $co['id'], 'companyName' => $co['name'] . '客户', 'is_del' => 0])
                    ->value('companyNo', '');

                if ($customer_code) $extend_insert_data['customer_code'] = $customer_code;
                else {
                    $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                    $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',在企业客户管理组织中找不到对应的企业' : '在企业客户管理组织中找不到对应的企业';
                }

            } else {
                $update_c_data['status'] = OIFCModel::$status_wait_confirm;
                $update_c_data['remark'] = isset($update_c_data['remark']) ? $update_c_data['remark'] . ',在企业客户管理组织中找不到平台名称对应的组织' : '在企业客户管理组织中找不到平台名称对应的组织';
            }

            //省市区
            if ($rs['addr']) {
                $p_c_a = get_address($rs['addr']);
                $addr[] = Db::name('province')->whereLike('name', '%' . $p_c_a['province'] . '%')->value('province_code');
                $addr[] = Db::name('city')->whereLike('name', '%' . $p_c_a['city'] . '%')->value('city_code');
                $addr[] = Db::name('area')->whereLike('name', '%' . $p_c_a['district'] . '%')->value('area_code');
                $extend_insert_data['addr_code'] = implode($addr, ',');
            }

            $order_import_from_c_extend_db->insertGetId($extend_insert_data);

            if (!isset($update_c_data['status'])) $update_c_data['status'] = OIFCModel::$status_wait_relation;

            $order_import_from_c_db
                ->where([
                    'id' => $rs['id'],
                    'is_del' => OIFCModel::$is_del_normal,
                    'status' => OIFCModel::$status_wait_validate
                ])->update($update_c_data);

            Db::commit();

            // 指令输出
            $output->writeln(date('Y-m-d H:i:s') . '|解析成功');

        } catch (Exception $exception) {
            Db::rollback();
            // 指令输出
            $output->writeln(date('Y-m-d H:i:s') . '|解析失败，' . $exception->getMessage() . '|' . $exception->getFile() . '|' . $exception->getLine());
        }


    }


}
